@@ -21,20 +21,23 @@ Rekor entries.
 The data format for the Merkle tree nodes is described in IETF's RFC 6962.
 """
 
+from __future__ import annotations
+
 import base64
 import hashlib
 import struct
+import typing
 from typing import List, Tuple
 
-from sigstore._internal.rekor import RekorEntry
+from sigstore._utils import HexStr
+from sigstore.errors import VerificationError
+
+if typing.TYPE_CHECKING:
+    from sigstore.models import LogEntry
 
 
-class InvalidInclusionProofError(Exception):
-    pass
-
-
-LEAF_HASH_PREFIX = 0
-NODE_HASH_PREFIX = 1
+_LEAF_HASH_PREFIX = 0
+_NODE_HASH_PREFIX = 1
 
 
 def _decomp_inclusion_proof(index: int, size: int) -> Tuple[int, int]:
@@ -81,21 +84,19 @@ def _chain_border_right(seed: bytes, hashes: List[str]) -> bytes:
 
 def _hash_children(lhs: bytes, rhs: bytes) -> bytes:
     pattern = f"B{len(lhs)}s{len(rhs)}s"
-    data = struct.pack(pattern, NODE_HASH_PREFIX, lhs, rhs)
+    data = struct.pack(pattern, _NODE_HASH_PREFIX, lhs, rhs)
     return hashlib.sha256(data).digest()
 
 
 def _hash_leaf(leaf: bytes) -> bytes:
     pattern = f"B{len(leaf)}s"
-    data = struct.pack(pattern, LEAF_HASH_PREFIX, leaf)
+    data = struct.pack(pattern, _LEAF_HASH_PREFIX, leaf)
     return hashlib.sha256(data).digest()
 
 
-def verify_merkle_inclusion(entry: RekorEntry) -> None:
-    """Verify the Merkle Inclusion Proof for a given Rekor entry"""
+def verify_merkle_inclusion(entry: LogEntry) -> None:
+    """Verify the Merkle Inclusion Proof for a given Rekor entry."""
     inclusion_proof = entry.inclusion_proof
-    if inclusion_proof is None:
-        raise InvalidInclusionProofError("Rekor entry has no inclusion proof")
 
     # Figure out which subset of hashes corresponds to the inner and border nodes.
     inner, border = _decomp_inclusion_proof(
@@ -104,8 +105,8 @@ def verify_merkle_inclusion(entry: RekorEntry) -> None:
 
     # Check against the number of hashes.
     if len(inclusion_proof.hashes) != (inner + border):
-        raise InvalidInclusionProofError(
-            f"Inclusion proof has wrong size: expected {inner + border}, got "
+        raise VerificationError(
+            f"inclusion proof has wrong size: expected {inner + border}, got "
             f"{len(inclusion_proof.hashes)}"
         )
 
@@ -119,12 +120,12 @@ def verify_merkle_inclusion(entry: RekorEntry) -> None:
         leaf_hash, inclusion_proof.hashes[:inner], inclusion_proof.log_index
     )
 
-    calc_hash: str = _chain_border_right(
-        intermediate_result, inclusion_proof.hashes[inner:]
-    ).hex()
+    calc_hash: HexStr = HexStr(
+        _chain_border_right(intermediate_result, inclusion_proof.hashes[inner:]).hex()
+    )
 
     if calc_hash != inclusion_proof.root_hash:
-        raise InvalidInclusionProofError(
-            f"Inclusion proof contains invalid root hash: expected {inclusion_proof}, calculated "
+        raise VerificationError(
+            f"inclusion proof contains invalid root hash: expected {inclusion_proof}, calculated "
             f"{calc_hash}"
         )
