@@ -19,26 +19,31 @@ Utilities for verifying signed certificate timestamps.
 import logging
 import struct
 from datetime import timezone
-from textwrap import dedent
 from typing import List, Optional
 
 from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec, rsa
-from cryptography.x509 import Certificate, ExtendedKeyUsage
+from cryptography.x509 import (
+    Certificate,
+    ExtendedKeyUsage,
+    ExtensionNotFound,
+    PrecertificateSignedCertificateTimestamps,
+)
 from cryptography.x509.certificate_transparency import (
     LogEntryType,
     SignedCertificateTimestamp,
 )
 from cryptography.x509.oid import ExtendedKeyUsageOID
 
-from sigstore._internal.ctfe import (
-    CTKeyring,
-    CTKeyringError,
-    CTKeyringLookupError,
+from sigstore._internal.trust import CTKeyring
+from sigstore._utils import (
+    KeyID,
+    cert_is_ca,
+    key_id,
 )
-from sigstore._utils import key_id
+from sigstore.errors import VerificationError
 
-logger = logging.getLogger(__name__)
+_logger = logging.getLogger(__name__)
 
 
 def _pack_signed_entry(
@@ -53,7 +58,7 @@ def _pack_signed_entry(
         cert_der = cert.public_bytes(encoding=serialization.Encoding.DER)
     elif sct.entry_type == LogEntryType.PRE_CERTIFICATE:
         if not issuer_key_id or len(issuer_key_id) != 32:
-            raise InvalidSctError("API misuse: issuer key ID missing")
+            raise VerificationError("API misuse: issuer key ID missing")
 
         # When dealing with a precertificate, our signed entry looks like this:
         #
@@ -65,7 +70,7 @@ def _pack_signed_entry(
         cert_der = cert.tbs_precertificate_bytes
         fields.append(issuer_key_id)
     else:
-        raise InvalidSctError(f"unknown SCT log entry type: {sct.entry_type!r}")
+        raise VerificationError(f"unknown SCT log entry type: {sct.entry_type!r}")
 
     # The `opaque` length is a u24, which isn't directly supported by `struct`.
     # So we have to decompose it into 3 bytes.
@@ -74,7 +79,9 @@ def _pack_signed_entry(
         struct.pack("!I", len(cert_der)),
     )
     if unused:
-        raise InvalidSctError(f"Unexpectedly large certificate length: {len(cert_der)}")
+        raise VerificationError(
+            f"Unexpectedly large certificate length: {len(cert_der)}"
+        )
 
     pack_format = pack_format.format(cert_der_len=len(cert_der))
     fields.extend((len1, len2, len3, cert_der))
@@ -85,20 +92,20 @@ def _pack_signed_entry(
 def _pack_digitally_signed(
     sct: SignedCertificateTimestamp,
     cert: Certificate,
-    issuer_key_id: Optional[bytes],
+    issuer_key_id: Optional[KeyID],
 ) -> bytes:
     """
     Packs the contents of `cert` (and some pieces of `sct`) into a structured
     blob, one that forms the signature body of the "digitally-signed" struct
     for an SCT.
 
-    The format of the digitally signed data is described in IETF's RFC 6962.
+    The format of the digitaly signed data is described in IETF's RFC 6962.
     """
 
     # No extensions are currently specified, so we treat the presence
     # of any extension bytes as suspicious.
     if len(sct.extension_bytes) != 0:
-        raise InvalidSctError("Unexpected trailing extension bytes")
+        raise VerificationError("Unexpected trailing extension bytes")
 
     # This constructs the "core" `signed_entry` field, which is either
     # the public bytes of the cert *or* the TBSPrecertificate (with some
@@ -125,7 +132,11 @@ def _pack_digitally_signed(
 
 
 def _is_preissuer(issuer: Certificate) -> bool:
-    ext_key_usage = issuer.extensions.get_extension_for_class(ExtendedKeyUsage)
+    try:
+        ext_key_usage = issuer.extensions.get_extension_for_class(ExtendedKeyUsage)
+    # If we do not have any EKU, we certainly do not have CT Ext
+    except ExtensionNotFound:
+        return False
 
     return ExtendedKeyUsageOID.CERTIFICATE_TRANSPARENCY in ext_key_usage.value
 
@@ -137,17 +148,58 @@ def _get_issuer_cert(chain: List[Certificate]) -> Certificate:
     return issuer
 
 
-class InvalidSctError(Exception):
+class UnexpectedSctCountException(Exception):
+    """
+    Number of percerts scts is wrong
+    """
+
     pass
 
 
+def _get_precertificate_signed_certificate_timestamps(
+    certificate: Certificate,
+) -> PrecertificateSignedCertificateTimestamps:
+    # Try to retrieve the embedded SCTs within the cert.
+    try:
+        precert_scts_extension = certificate.extensions.get_extension_for_class(
+            PrecertificateSignedCertificateTimestamps
+        ).value
+    except ExtensionNotFound:
+        raise ValueError(
+            "No PrecertificateSignedCertificateTimestamps found for the certificate"
+        )
+
+    if len(precert_scts_extension) != 1:
+        raise UnexpectedSctCountException(
+            f"Unexpected embedded SCT count in response: {len(precert_scts_extension)} != 1"
+        )
+    return precert_scts_extension
+
+
+def _cert_is_ca(cert: Certificate) -> bool:
+    _logger.debug(f"Found {cert.subject} as issuer, verifying if it is a ca")
+    try:
+        cert_is_ca(cert)
+    except VerificationError as e:
+        _logger.debug(f"Invalid {cert.subject}: failed to validate as a CA: {e}")
+        return False
+    return True
+
+
 def verify_sct(
     sct: SignedCertificateTimestamp,
     cert: Certificate,
     chain: List[Certificate],
     ct_keyring: CTKeyring,
 ) -> None:
-    """Verify a signed certificate timestamp"""
+    """
+    Verify a signed certificate timestamp.
+
+    An SCT is verified by reconstructing its "digitally-signed" payload
+    and verifying that the signature provided in the SCT is valid against
+    one of the keys present in the CT keyring (i.e., the keys used by the CT
+    log to sign SCTs).
+    """
 
     issuer_key_id = None
     if sct.entry_type == LogEntryType.PRE_CERTIFICATE:
@@ -155,11 +207,17 @@ def verify_sct(
         # find its issuer in the chain and calculate a hash over
         # its public key information, as part of the "binding" proof
         # that ties the issuer to the final certificate.
-        issuer_pubkey = _get_issuer_cert(chain).public_key()
+        issuer_cert = _get_issuer_cert(chain)
+        issuer_pubkey = issuer_cert.public_key()
+
+        if not _cert_is_ca(issuer_cert):
+            raise VerificationError(
+                f"SCT verify: Invalid issuer pubkey basicConstraint (not a CA): {issuer_pubkey}"
+            )
 
         if not isinstance(issuer_pubkey, (rsa.RSAPublicKey, ec.EllipticCurvePublicKey)):
-            raise InvalidSctError(
-                f"invalid issuer pubkey format (not ECDSA or RSA): {issuer_pubkey}"
+            raise VerificationError(
+                f"SCT verify: invalid issuer pubkey format (not ECDSA or RSA): {issuer_pubkey}"
             )
 
         issuer_key_id = key_id(issuer_pubkey)
@@ -167,43 +225,19 @@ def verify_sct(
     digitally_signed = _pack_digitally_signed(sct, cert, issuer_key_id)
 
     if not isinstance(sct.signature_hash_algorithm, hashes.SHA256):
-        raise InvalidSctError(
+        raise VerificationError(
             "Found unexpected hash algorithm in SCT: only SHA256 is supported "
             f"(expected {hashes.SHA256}, got {sct.signature_hash_algorithm})"
         )
 
     try:
-        logger.debug(f"attempting to verify SCT with key ID {sct.log_id.hex()}")
+        _logger.debug(f"attempting to verify SCT with key ID {sct.log_id.hex()}")
         # NOTE(ww): In terms of the DER structure, the SCT's `LogID` contains a
         # singular `opaque key_id[32]`. Cryptography's APIs don't bother
         # to expose this trivial single member, so we use the `log_id`
         # attribute directly.
         ct_keyring.verify(
-            key_id=sct.log_id, signature=sct.signature, data=digitally_signed
+            key_id=KeyID(sct.log_id), signature=sct.signature, data=digitally_signed
         )
-    except CTKeyringLookupError as exc:
-        # We specialize this error case, since it usually indicates one of
-        # two conditions: either the current sigstore client is out-of-date,
-        # or that the SCT is well-formed but invalid for the current configuration
-        # (indicating that the user has asked for the wrong instance).
-        #
-        # TODO(ww): Longer term, this should be specialized elsewhere.
-        raise InvalidSctError(
-            dedent(
-                f"""
-                Invalid key ID in SCT: not found in current keyring.
-
-                This may be a result of an outdated `sigstore` installation.
-
-                Consider upgrading with:
-
-                    python -m pip install --upgrade sigstore
-
-                Additional context:
-
-                {exc}
-                """
-            ),
-        )
-    except CTKeyringError as exc:
-        raise InvalidSctError from exc
+    except VerificationError as exc:
+        raise VerificationError(f"SCT verify failed: {exc}")
